﻿using System;
using System.Text.RegularExpressions;

namespace REFOR.BL
{
    /// <summary>
    /// Operaciones relacionadas con el CCC y el IBAN
    /// </summary>
    public class Cuenta
    {
        /// <summary>
        /// Calcula los digitos de control de un CCC.
        /// </summary>
        /// <param name="entidad">Entidad</param>
        /// <param name="sucursal">Sucursal</param>
        /// <param name="cuenta">Cuenta</param>
        /// <returns>El CCC con los digitos de control</returns>
        public static string CalculaCCC(string entidad, string sucursal, string cuenta)
        {
            double resto;
            string CCC;

            CCC = entidad.Trim().PadLeft(4, '0') + sucursal.Trim().PadLeft(4, '0') + "00" + cuenta.Trim().PadLeft(10, '0');
            if (!double.TryParse(CCC, out resto))
            {
                throw new Exception("Alguno de los parametros no está compuesto solo por números");
            }

            return CalculaCCC(CCC);
        }

        /// <summary>
        /// Calcula los digitos de control de un CCC.
        /// </summary>
        /// <param name="CCC">El CCC del que se calculan los dígitos de control.</param>
        /// <returns>El CCC con los digitos de control</returns>
        public static string CalculaCCC(string CCC)
        {
            CCC = CCC.Trim();

            int restoEntidadSucursal, restoCuenta;
            double test;

            if (!double.TryParse(CCC, out test))
            {
                throw new Exception("La cuenta no está formada exclusivamente por números");
            }

            if (CCC.Trim().Length != 20)
            {
                throw new Exception("La longuitud de la cuenta no es correcta (debe ser 20)");
            }

            int[] pesos = { 6, 3, 7, 9, 10, 5, 8, 4, 2, 1 };

            int sumaPesos = 0;
            for (int i = 0; i <= 7; i++)      // CALCULAMOS EL DIGITO DE CONTROL DE ENTIDAD+SUCURSAL
            {
                sumaPesos += Convert.ToInt16(CCC.Substring(7 - i, 1)) * pesos[i];
            }

            restoEntidadSucursal = 11 - (sumaPesos % 11);
            if (restoEntidadSucursal == 10)
                restoEntidadSucursal = 1;
            else
                if (restoEntidadSucursal == 11)
                    restoEntidadSucursal = 0;

            sumaPesos = 0;
            for (int i = 0; i <= 9; i++)      // CALCULAMOS EL DIGITO DE CONTROL DE ENTIDAD+SUCURSAL
            {
                sumaPesos += Convert.ToInt16(CCC.Substring(10, 10).Substring(9 - i, 1)) * pesos[i];
            }

            //restoCuenta = 11 - Math.IEEERemainder(Convert.ToDouble(sumaPesos), 11);
            restoCuenta = 11 - (sumaPesos % 11);
            if (restoCuenta == 10)
                restoCuenta = 1;
            else
                if (restoCuenta == 11)
                    restoCuenta = 0;

            CCC = CCC.Substring(0, 8) + restoEntidadSucursal.ToString() + restoCuenta.ToString() + CCC.Substring(10, 10);

            return CCC;
        }

        /// <summary>
        /// Comprueba un CCC.
        /// </summary>
        /// <param name="CCC">El CCC a comprobar.</param>
        /// <returns>Cierto si el CCC es correcto, falso en caso contrario</returns>
        public static bool CompruebaCCC(string CCC)
        {
            CCC = CCC.Trim();
            string CopiaCCC;
            try
            {
                CopiaCCC = CalculaCCC(CCC);
            }
            catch
            {
                return false;       // si se produce cualquier excepcion, es que no es correcto
            }
            if (CCC.Equals(CopiaCCC))
                return true;
            else
                return false;
        }

        /// <summary>
        /// Comprueba un CCC.
        /// </summary>
        /// <param name="CCC">El CCC a comprobar.</param>
        /// <param name="mensaje">La razón por la que el CCC no es correcto.</param>
        /// <returns>
        /// Cierto si el CCC es correcto, falso en caso contrario
        /// </returns>
        public static bool CompruebaCCC(string CCC, out string mensaje)
        {
            CCC = CCC.Trim();
            string CopiaCCC;
            try
            {
                CopiaCCC = CalculaCCC(CCC);
            }
            catch (Exception ex)
            {
                mensaje = ex.Message;
                return false;       // si se produce cualquier excepcion, es que no es correcto
            }
            if (CCC.Equals(CopiaCCC))
            {
                mensaje = "OK";
                return true;
            }
            else
            {
                mensaje = "Los digitos de control calculados " + CopiaCCC.Substring(8, 2) + " no coinciden con los introducidos " + CCC.Substring(8, 2);
                return false;
            }
        }


        /// <summary>
        /// Calcula los digitos de control de un IBAN.
        /// </summary>
        /// <param name="IBAN">Ell IBAN del que se calculan los digitos de control.</param>
        /// <returns>
        /// El IBAN con los digitos de control
        /// </returns>
        public static string CalculaIBAN(string IBAN)
        {

            IBAN = IBAN.Trim().ToUpper();

            Int64 resto;

            double test;

            if (!Regex.IsMatch(IBAN.Substring(0, 2), "\\w"))
            {
                throw new Exception("El IBAN no comienza por 2 letras");
            }

            if (IBAN.Substring(0, 2).Equals("ES"))
            {
                if (!double.TryParse(IBAN.Substring(2), out test))
                    throw new Exception("El IBAN no está formada exclusivamente por números a partir de la 3 posición");

                if (IBAN.Length != 24)
                    throw new Exception("La longuitud del IBAN español no es correcta (debe ser 24)");

                if (!CompruebaCCC(IBAN.Substring(4, 20)))
                    throw new Exception("El CCC incluido en el IBAN no es correcto");
            }

            string copiaIban = string.Empty;
            string IBANinvertido = IBAN.Substring(4) + IBAN.Substring(0, 2) + "00";   // PONEMOS LAS 2 LETRAS AL FINAL Y AÑADIMOS 00


            for (int i = 0; i <= IBANinvertido.Length - 1; i++)      // CALCULAMOS LOS DIGITOS DE CONTROL DEL IBAN
            {
                copiaIban += pesos_IBAN(IBANinvertido.Substring(i, 1));
            }

            // int64 puede contener 18 dígitos pero solo tomamos 9 para mayor seguridad
            Int64 restoCalculado = 0;
            while (copiaIban.Length > 9)
            {
                restoCalculado = Convert.ToInt64(copiaIban.Substring(0, 9)) % 97;
                copiaIban = copiaIban.Substring(9);     // le quitamos las primeras 9 posiciones sobre las que hemos calculado el resto
                copiaIban = restoCalculado.ToString() + copiaIban;  // añadimos el resto calculado por delante 
            }

            resto = 98 - (Convert.ToInt64(copiaIban) % 97);

            return IBAN.Substring(0, 2) + resto.ToString().PadLeft(2, '0') + IBAN.Substring(4);
        }


        /// <summary>
        /// Calcula los digitos de control de un IBAN.
        /// </summary>
        /// <param name="IBAN">El IBAN del que se calculan los digitos de control.</param>
        /// <param name="mensaje">El mensaje que indica la causa del error.</param>
        /// <returns>
        /// El IBAN con los digitos de control
        /// </returns>
        public static string CalculaIBANSinExcepcion(string IBAN, out string mensaje)
        {

            IBAN = IBAN.Trim().ToUpper();

            Int64 resto;

            double test;

            if (!Regex.IsMatch(IBAN.Substring(0, 2), "\\w"))
            {
                mensaje = "El IBAN no comienza por 2 letras";
                return string.Empty;
            }

            if (IBAN.Substring(0, 2).Equals("ES"))
            {
                if (!double.TryParse(IBAN.Substring(2), out test))
                {
                    mensaje = "El IBAN no está formada exclusivamente por números a partir de la 3 posición";
                    return string.Empty;
                }

                if (IBAN.Length != 24)
                {
                    mensaje = "La longuitud del IBAN español no es correcta (debe ser 24)";
                    return string.Empty;
                }

                string copiaCCC = CalculaCCC(IBAN.Substring(4, 20));
                if (!CompruebaCCC(IBAN.Substring(4, 20)))
                {
                    mensaje = "El CCC incluido en el IBAN no es correcto. El correcto es " + copiaCCC;
                    return string.Empty;
                }
            }

            string copiaIban = string.Empty;
            string IBANinvertido = IBAN.Substring(4) + IBAN.Substring(0, 2) + "00";   // PONEMOS LAS 2 LETRAS AL FINAL Y AÑADIMOS 00


            for (int i = 0; i <= IBANinvertido.Length - 1; i++)      // CALCULAMOS LOS DIGITOS DE CONTROL DEL IBAN
            {
                copiaIban += pesos_IBAN(IBANinvertido.Substring(i, 1));
            }

            // int64 puede contener 18 dígitos pero solo tomamos 9 para mayor seguridad
            Int64 restoCalculado = 0;
            while (copiaIban.Length > 9)
            {
                restoCalculado = Convert.ToInt64(copiaIban.Substring(0, 9)) % 97;
                copiaIban = copiaIban.Substring(9);     // le quitamos las primeras 9 posiciones sobre las que hemos calculado el resto
                copiaIban = restoCalculado.ToString() + copiaIban;  // añadimos el resto calculado por delante 
            }

            resto = 98 - (Convert.ToInt64(copiaIban) % 97);

            mensaje = "OK";
            return IBAN.Substring(0, 2) + resto.ToString().PadLeft(2, '0') + IBAN.Substring(4);
        }

        /// <summary>
        /// Pesos_s the IBAN.
        /// </summary>
        /// <param name="letra">The letra.</param>
        /// <returns>El peso de la letra para calculo de IBAN</returns>
        private static string pesos_IBAN(string letra)
        {
            letra = letra.ToUpper();

            if (Regex.IsMatch(letra, "\\d"))
                return letra;

            string resultado;

            switch (letra)
            {
                case "A":
                    resultado = "10";
                    break;
                case "B":
                    resultado = "11";
                    break;
                case "C":
                    resultado = "12";
                    break;
                case "D":
                    resultado = "13";
                    break;
                case "E":
                    resultado = "14";
                    break;
                case "F":
                    resultado = "15";
                    break;
                case "G":
                    resultado = "16";
                    break;
                case "H":
                    resultado = "17";
                    break;
                case "I":
                    resultado = "18";
                    break;
                case "J":
                    resultado = "19";
                    break;
                case "K":
                    resultado = "20";
                    break;
                case "L":
                    resultado = "21";
                    break;
                case "M":
                    resultado = "22";
                    break;
                case "N":
                    resultado = "23";
                    break;
                case "O":
                    resultado = "24";
                    break;
                case "P":
                    resultado = "25";
                    break;
                case "Q":
                    resultado = "26";
                    break;
                case "R":
                    resultado = "27";
                    break;
                case "S":
                    resultado = "28";
                    break;
                case "T":
                    resultado = "29";
                    break;
                case "U":
                    resultado = "30";
                    break;
                case "V":
                    resultado = "31";
                    break;
                case "W":
                    resultado = "32";
                    break;
                case "X":
                    resultado = "33";
                    break;
                case "Y":
                    resultado = "34";
                    break;
                case "Z":
                    resultado = "35";
                    break;
                default:
                    resultado = "00";
                    break;
            }
            return resultado;
        }

        /// <summary>
        /// Comprueba el IBAN.
        /// </summary>
        /// <param name="IBAN">El IBAN a comprobar.</param>
        /// <returns>true si el IBAN es correcto, false en caso contrario</returns>
        public static bool CompruebaIBAN(string IBAN)
        {
            IBAN = IBAN.Trim().ToUpper();
            string CopiaIBAN;
            try
            {
                CopiaIBAN = CalculaIBAN(IBAN);
            }
            catch
            {
                return false;       // si se produce cualquier excepcion, es que no es correcto
            }
            if (IBAN.Equals(CopiaIBAN))
                return true;
            else
                return false;
        }

        /// <summary>
        /// Comprueba el IBAN.
        /// </summary>
        /// <param name="IBAN">El IBAN a comprobar.</param>
        /// <param name="mensaje">La razón por la que el IBAN no es correcto.</param>
        /// <returns>
        /// true si el IBAN es correcto, false en caso contrario
        /// </returns>
        public static bool CompruebaIBAN(string IBAN, out string mensaje)
        {
            IBAN = IBAN.Trim().ToUpper();
            string CopiaIBAN;
            try
            {
                CopiaIBAN = CalculaIBANSinExcepcion(IBAN, out mensaje);
            }
            catch (Exception ex)
            {
                mensaje = ex.Message;
                return false;       // si se produce cualquier excepcion, es que no es correcto
            }
            if (IBAN.Equals(CopiaIBAN))
            {
                mensaje = "OK";
                return true;
            }
            else
            {
                if (mensaje == "OK")
                    mensaje = "Los digitos de control calculados " + CopiaIBAN.Substring(2, 2) + " no coinciden con los introducidos " + IBAN.Substring(2, 2);
                return false;
            }
        }

        /// <summary>
        /// Devuelve el IBAN a partir del CCC.
        /// </summary>
        /// <param name="CCC">El CCC.</param>
        /// <returns>El IBAN obtenido</returns>
        public static string CCC_to_IBAN(string CCC)
        {
            if (!CompruebaCCC(CCC))
                throw new Exception("CCC no es correcto");

            return CalculaIBAN("ES00" + CCC);
        }
    }
}
