﻿using System;
using System.Text.RegularExpressions;

namespace REFOR.BL
{
    /// <summary>
    /// Validaciones de NIF/CIF/NIE
    /// </summary>
    public class VatNumber
    {
        string idVatNumber;
        bool isPhysicalPerson;
        bool isJuridicalPerson;
        bool isValid;
        string validationReason;
        string typeOfDocument;
        string juridicalStatus;

        /// <summary>
        /// Initializes a new instance of the <see cref="VatNumber"/> class.
        /// </summary>
        public VatNumber()
        {
        }

        /// <summary>
        /// Vat number 
        /// </summary>
        public string IDVatNumber
        {
            get
            {
                return idVatNumber;
            }
            set
            {
                idVatNumber = value;
                Validate();
            }
        }
        /// <summary>
        /// Is a valid NIF/CIF/NIE
        /// </summary>
        public bool IsValid
        {
            get
            {
                return isValid;
            }
        }

        /// <summary>
        /// Reason fon non valid
        /// </summary>
        public string ValidationReason
        {
            get
            {
                return validationReason;
            }
        }

        /// <summary>
        /// Return true in case the identification match a juridical person
        /// </summary>
        public bool IsJuridicalPerson
        {
            get
            {
                return isJuridicalPerson;
            }
        }

        /// <summary>
        /// Return true in case the identification match a physical person
        /// </summary>
        public bool IsPhysicalPerson
        {
            get
            {
                return isPhysicalPerson;
            }
        }

        /// <summary>
        /// Type of document NIF/CIF/NIE
        /// </summary>
        public string TypeOfDocument
        {
            get
            {
                return typeOfDocument;
            }
        }

        /// <summary>
        /// Juridical status
        /// </summary>
        public string JuridicalStatus
        {
            get
            {
                return juridicalStatus;
            }
        }

        /// <summary>
        /// Returns true if the vat number is correct
        /// </summary>
        public void Validate()
        {
            isValid = true;
            isPhysicalPerson = false;
            isJuridicalPerson = false;
            validationReason = string.Empty;
            typeOfDocument = string.Empty;
            juridicalStatus = string.Empty;
            //
            // Valid chains. 
            // a cif (nif for letters KLM)  starting with A,B,C,D,E,F,G,H,N,P,Q,S,K,L,M plus 7 digit and a letter or digit
            // a dni starting with 8 digits plus a letter
            // a nie starting with X plus 7 digit and a letter or digit
            // a nie starting with T plus 8 digit or letters. WARNING there is no check digit
            Regex rge = new Regex(@"[ABCDEFGHJKLMNPQRSUVWXYZ]\d{7}[0-9A-Z]|\d{8}[A-Z]|[T][0-9A-Z]{8}");
            if (rge.IsMatch(idVatNumber) == false)
            {
                isValid = false;
                validationReason = "El formato del NIF/CIF/NIE no es correcto en su estructura";
                return;
            }
            char[] letters = idVatNumber.ToCharArray();
            // if the function arrives here is because the lenght is 9, 
            // starts with a letter/digit plus 7 digit plus another letter/digit
            // sum of weights
            int sum = 0;
            sum = Convert.ToInt32(letters[2].ToString());
            sum += Convert.ToInt32(letters[4].ToString());
            sum += Convert.ToInt32(letters[6].ToString());
            sum += ((Convert.ToInt32(letters[1].ToString()) * 2) % 10) + ((Convert.ToInt32(letters[1].ToString()) * 2) / 10);
            sum += ((Convert.ToInt32(letters[3].ToString()) * 2) % 10) + ((Convert.ToInt32(letters[3].ToString()) * 2) / 10);
            sum += ((Convert.ToInt32(letters[5].ToString()) * 2) % 10) + ((Convert.ToInt32(letters[5].ToString()) * 2) / 10);
            sum += ((Convert.ToInt32(letters[7].ToString()) * 2) % 10) + ((Convert.ToInt32(letters[7].ToString()) * 2) / 10);
            // control digit
            int digit = 10 - (sum % 10);
            char controlLetter = (char)(64 + digit);
            if (digit == 10)
                digit = 0;
            char[] dniLetters = "TRWAGMYFPDXBNJZSQVHLCKE".ToCharArray();
            // physical or juridical person
            if ("0123456789KLMTX".IndexOf(letters[0]) > 0)
            {
                isPhysicalPerson = true;
                isJuridicalPerson = false;
                switch (letters[0])
                {
                    case 'K':
                        juridicalStatus = "Españoles menores de 14 años y extranjeros menores de 18";
                        break;
                    case 'L':
                        juridicalStatus = "Españoles mayores de 14 años residentes en el extranjero y que se trasladan por tiempo inferior a seis meses a España";
                        break;
                    case 'M':
                        juridicalStatus = "Extranjeros mayores de 18 años";
                        break;
                    case 'T':
                        juridicalStatus = "Extranjeros sin identificación";
                        break;
                    case 'X':
                    case 'Y':
                    case 'Z':
                        juridicalStatus = "Extranjeros con NIE";
                        break;
                    default:
                        juridicalStatus = "Persona fisica española";
                        break;
                }

            }
            else
            {
                isPhysicalPerson = false;
                isJuridicalPerson = true;
                switch (letters[0])
                {
                    case 'A':
                        juridicalStatus = "Sociedades Anónimas";
                        break;
                    case 'B':
                        juridicalStatus = "Sociedades de Responsabilidad Limitada";
                        break;
                    case 'C':
                        juridicalStatus = "Sociedades Colectivas";
                        break;
                    case 'D':
                        juridicalStatus = "Sociedades Comanditarias";
                        break;
                    case 'E':
                        juridicalStatus = "Comunidades de bienes y herecias yacentes";
                        break;
                    case 'F':
                        juridicalStatus = "Sociedades Cooperativas";
                        break;
                    case 'G':
                        juridicalStatus = "Asociaciones";
                        break;
                    case 'H':
                        juridicalStatus = "Comunidades de propietarios en régimen de propiedad horizontal";
                        break;
                    case 'J':
                        juridicalStatus = "Sociedades civiles, con o sin personalidad jurídica";
                        break;
                    case 'N':
                        juridicalStatus = "Entidades extranjera";
                        break;
                    case 'P':
                        juridicalStatus = "Corporaciones locales";
                        break;
                    case 'Q':
                        juridicalStatus = "Organismos públicos";
                        break;
                    case 'R':
                        juridicalStatus = "Congregaciones e Instituciones religiosas";
                        break;
                    case 'S':
                        juridicalStatus = "Organos de la Administración del Estado y Comunidades Autónomas";
                        break;
                    case 'U':
                        juridicalStatus = "Uniones Temporales de Empresas";
                        break;
                    case 'V':
                        juridicalStatus = "Otros tipos no definidos en el resto de claves";
                        break;
                    case 'W':
                        juridicalStatus = "Establecimiento permanente de una entidad no residente";
                        break;
                    default:
                        juridicalStatus = "Error";
                        break;
                }

            }
            Regex rge1 = new Regex(@"[ABCDEFGHJNPQRSUVW]\d{7}[0-9A-Z]");
            if (rge1.IsMatch(idVatNumber) == true)
            {
                typeOfDocument = "CIF";
                if ((letters[8] == controlLetter) || (letters[8] == Convert.ToChar(digit.ToString())))
                    isValid = true;
                else
                {
                    isValid = false;
                    validationReason = "Digito de control " + letters[8] + " no coincide con el calculado " + controlLetter + "/" + digit;
                }
                return;
            }
            Regex rge2 = new Regex(@"[KLM]\d{7}[0-9A-Z]");
            if (rge2.IsMatch(idVatNumber) == true)
            {
                typeOfDocument = "NIF";
                if ((letters[8] == controlLetter))
                    isValid = true;
                else
                {
                    isValid = false;
                    validationReason = "Digito de control " + letters[8] + " no coincide con el calculado " + controlLetter;
                }
                return;
            }
            Regex rge3 = new Regex(@"[T][0-9A-Z]{8}");
            if (rge3.IsMatch(idVatNumber) == true)
            {
                typeOfDocument = "NIE";
                isValid = true;
                return;
            }
            Regex rge4 = new Regex(@"[XYZ]\d{7}[0-9A-Z]");
            if (rge4.IsMatch(idVatNumber) == true)
            {

                int dniNumber;
                switch (idVatNumber[0])
                {
                    case 'Y':
                        dniNumber = (Convert.ToInt32(("1"+idVatNumber.Substring(1, 7))) % 23);
                        break;
                    case 'Z':
                        dniNumber = (Convert.ToInt32(("2"+idVatNumber.Substring(1, 7))) % 23);
                        break;
                    default:        // la x es como un cero 
                        dniNumber = (Convert.ToInt32(idVatNumber.Substring(1, 7)) % 23);
                        break;
                }

                typeOfDocument = "NIE";
                if (letters[8] == dniLetters[dniNumber])
                    isValid = true;
                else
                {
                    isValid = false;
                    validationReason = "Digito de control " + letters[8] + " no coincide con el calculado " + dniLetters[dniNumber];
                }
                return;
            }
            Regex rge5 = new Regex(@"\d{8}[A-Z]");
            if (rge5.IsMatch(idVatNumber) == true)
            {
                int dniNumber = (Convert.ToInt32(idVatNumber.Substring(0, 8)) % 23);
                typeOfDocument = "DNI";
                if (letters[8] == dniLetters[dniNumber])
                    isValid = true;
                else
                {
                    isValid = false;
                    validationReason = "Digito de control " + letters[8] + " no coincide con el calculado " + dniLetters[dniNumber];
                }
                return;
            }
            isValid = false;
            validationReason = "Formato desconocido";
            return;
        }


    }
}
