﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;

/// <summary>
/// Parte del data layer
/// </summary>
namespace REFOR.DL
{
    /// <summary>
    /// Maneja todas las operaciones de la clase
    /// </summary>
    public class LineaPresupuestoRepository : IDisposable
    {
        /// <summary>
        /// El contexto utilizado para interactuar con la base de datos
        /// </summary>
        private ReformasEntities context = new ReformasEntities();

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Texto">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<LineaPresupuesto> GetLineasPresupuesto(int PresupuestoID)
        {
            try
            {
                return context.LineasPresupuesto.Include("Producto").Include("Producto.TipoProducto").Where(r => r.PresupuestoID == PresupuestoID).OrderBy(r => r.Orden).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }


        /// <summary>
        /// Inserta un nuevo elemento.
        /// </summary>
        /// <param name="Texto">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Insertar(int ProductoID, decimal Cantidad, string Texto, int PresupuestoID )
        {

            // averiaguamos el ultimo numero de orden utilizado para este presupuesto
            int orden = UltimoNumeroOrden(PresupuestoID);
            // tomamos los datos del producto 
            Producto producto = (new ProductoRepository()).ObtenerPorID(ProductoID);

            LineaPresupuesto lineaPresupuesto = new LineaPresupuesto();
            lineaPresupuesto.LineaID = 0;
            lineaPresupuesto.Orden = (orden + 10);
            lineaPresupuesto.ProductoID = ProductoID;
            lineaPresupuesto.PresupuestoID = PresupuestoID;
            lineaPresupuesto.Cantidad = Cantidad;
            lineaPresupuesto.Precio = producto.Precio;
            lineaPresupuesto.Coste = producto.Coste;
            lineaPresupuesto.Texto = Texto;

            context.LineasPresupuesto.Add(lineaPresupuesto);
            int respuesta = SaveChanges();

            (new PresupuestoRepository()).Recalcular(PresupuestoID);

            return respuesta;
        }

        /// <summary>
        /// Ultimoes the numero orden.
        /// </summary>
        /// <param name="PresupuestoID">The presupuesto identifier.</param>
        /// <returns>El ultimo numero de orden</returns>
        /// <exception cref="System.NotImplementedException"></exception>
        private int UltimoNumeroOrden(int PresupuestoID)
        {
            try
            {
                LineaPresupuesto ultimaLinea = context.LineasPresupuesto.Where(r => r.PresupuestoID == PresupuestoID).OrderBy(r => r.Orden).ToList().Last();
                return ultimaLinea.Orden;
            }
            catch
            {
                return 0;
            }
        }

        /// <summary>
        /// Obtiene un elemento por su identificativo.
        /// </summary>
        /// <param name="LineaID">La clave a buscar.</param>
        /// <returns>Lista de elementos solicitados</returns>
        public LineaPresupuesto ObtenerPorID(int LineaID)
        {
            return context.LineasPresupuesto.Find(LineaID);
        }

        /// <summary>
        /// Elimina un elemento de la tabla.
        /// </summary>
        /// <param name="LineaID">La clave a buscar.</param>
        /// <returns>registros afectados</returns>
        public int Eliminar(int LineaID)
        {
            LineaPresupuesto lineaPresupuesto = ObtenerPorID(LineaID);
            if (lineaPresupuesto == null)
                return 0;

            int PresupuestoID = lineaPresupuesto.PresupuestoID;

            context.LineasPresupuesto.Remove(lineaPresupuesto);
            int resultado = SaveChanges();

            (new PresupuestoRepository()).Recalcular(PresupuestoID);
            (new PresupuestoRepository()).Reordenar(PresupuestoID);

            return resultado;
        }

        /// <summary>
        /// Actualiza un elemento de la tabla.
        /// </summary>
        /// <param name="LineaID">La clave a buscar.</param>
        /// <param name="Texto">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Actualizar( int LineaID ,int Orden, int ProductoID, decimal Cantidad, string Texto)
        {
            LineaPresupuesto lineaPresupuesto = ObtenerPorID(LineaID);
            if (lineaPresupuesto == null)
                return 0;
            
            // tomamos los datos del producto 
            Producto producto = (new ProductoRepository()).ObtenerPorID(ProductoID);
            
            lineaPresupuesto.Orden = Orden;
            lineaPresupuesto.ProductoID = ProductoID;
            lineaPresupuesto.Cantidad = Cantidad;
            lineaPresupuesto.Precio = producto.Precio;
            lineaPresupuesto.Coste = producto.Coste;
            lineaPresupuesto.Texto = Texto.Trim();


            int resultado = SaveChanges();

            (new PresupuestoRepository()).Recalcular(lineaPresupuesto.PresupuestoID);
            (new PresupuestoRepository()).Reordenar(lineaPresupuesto.PresupuestoID);

            return resultado;
        }

        /// <summary>
        /// Guarda los campos en la tabla.
        /// </summary>
        /// <returns>registros afectados</returns>
        /// <exception cref="System.Exception"></exception>
        public int SaveChanges()
        {
            try
            {
                return context.SaveChanges();
            }
            catch (Exception ex)
            {
                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        private bool disposedValue = false;

        /// <summary>
        /// Releases unmanaged and - optionally - managed resources.
        /// </summary>
        /// <param name="disposing"><c>true</c> to release both managed and unmanaged resources; <c>false</c> to release only unmanaged resources.</param>
        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposedValue)
            {
                if (disposing)
                {
                    context.Dispose();
                }
            }
            this.disposedValue = true;
        }

        /// <summary>
        /// Realiza tareas definidas por la aplicación asociadas a la liberación o al restablecimiento de recursos no administrados.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

    }
}
